package sun.java2d.pipe;

import java.awt.Rectangle;
import java.awt.Shape;
import sun.java2d.SunGraphics2D;

public class SpanClipRenderer implements CompositePipe
{
    CompositePipe outpipe;
    static Class RegionClass;
    static Class RegionIteratorClass;
    
    static native void initIDs(Class p0, Class p1);
    
    public SpanClipRenderer(CompositePipe outpipe) {
        this.outpipe = outpipe;
    }
    
    @Override
    public Object startSequence(SunGraphics2D sunGraphics2D, Shape shape, Rectangle rectangle, int[] array) {
        return new SpanClipRenderer.SCRcontext(sunGraphics2D.clipRegion.getIterator(), this.outpipe.startSequence(sunGraphics2D, shape, rectangle, array));
    }
    
    @Override
    public boolean needTile(Object o, int n, int n2, int n3, int n4) {
        return this.outpipe.needTile(((SpanClipRenderer.SCRcontext)o).outcontext, n, n2, n3, n4);
    }
    
    public void renderPathTile(Object o, byte[] array, int n, int n2, int n3, int n4, int n5, int n6, ShapeSpanIterator shapeSpanIterator) {
        this.renderPathTile(o, array, n, n2, n3, n4, n5, n6);
    }
    
    @Override
    public void renderPathTile(Object o, byte[] tile, int n, int n2, int n3, int n4, int n5, int n6) {
        SpanClipRenderer.SCRcontext scRcontext = (SpanClipRenderer.SCRcontext)o;
        RegionIterator copy = scRcontext.iterator.createCopy();
        int[] band = scRcontext.band;
        band[0] = n3;
        band[1] = n4;
        band[2] = n3 + n5;
        band[3] = n4 + n6;
        if (tile == null) {
            int n7 = n5 * n6;
            tile = scRcontext.tile;
            if (tile != null && tile.length < n7) {
                tile = null;
            }
            if (tile == null) {
                tile = new byte[n7];
                scRcontext.tile = tile;
            }
            n = 0;
            n2 = n5;
            this.fillTile(copy, tile, n, n2, band);
        }
        else {
            this.eraseTile(copy, tile, n, n2, band);
        }
        if (band[2] > band[0] && band[3] > band[1]) {
            n += (band[1] - n4) * n2 + (band[0] - n3);
            this.outpipe.renderPathTile(scRcontext.outcontext, tile, n, n2, band[0], band[1], band[2] - band[0], band[3] - band[1]);
        }
    }
    
    public native void fillTile(RegionIterator p0, byte[] p1, int p2, int p3, int[] p4);
    
    public native void eraseTile(RegionIterator p0, byte[] p1, int p2, int p3, int[] p4);
    
    @Override
    public void skipTile(Object o, int n, int n2) {
        this.outpipe.skipTile(((SpanClipRenderer.SCRcontext)o).outcontext, n, n2);
    }
    
    @Override
    public void endSequence(Object o) {
        this.outpipe.endSequence(((SpanClipRenderer.SCRcontext)o).outcontext);
    }
    
    static {
        SpanClipRenderer.RegionClass = Region.class;
        SpanClipRenderer.RegionIteratorClass = RegionIterator.class;
        initIDs(SpanClipRenderer.RegionClass, SpanClipRenderer.RegionIteratorClass);
    }
    
    class SCRcontext
    {
        RegionIterator iterator;
        Object outcontext;
        int[] band;
        byte[] tile;
        
        public SCRcontext(RegionIterator iterator, Object outcontext) {
            this.iterator = iterator;
            this.outcontext = outcontext;
            this.band = new int[4];
        }
    }
}
